 /* ATB_I2C_Strommessung_Prg2.c
 * Created: 07.04.2018 19:00:28 Author : Hans-Joachim Seeger */ 
 
// Meine Hardware: NT2, Board1 mit ATm 1284p, LCD Display, INA219 Modul

#include <stdbool.h>
#include <avr/pgmspace.h>
#include "main.h"					// Angabe Adressen
#include <util/delay.h>
#include "i2clcd.h"
#include "i2cmaster.h"
#include "avr/io.h"
#include "util/delay.h"
#include "avr/interrupt.h"
#include "stdlib.h"

#define calibration 0x107E			// Cal Reg 0x107E

uint16_t msb_strom;        			// Oberes Strom-Byte
uint16_t lsb_strom;        			// Unteres Strom-Byte
uint16_t strom_wrd;       			// Ganzes Strom-Wort
uint16_t strom_anz;       			// Ganzes Strom-Anzeige

uint16_t msb_spannung;        		// Oberes Spannungs-Byte
uint16_t lsb_spannung;        		// Unteres Spannungs-Byte
uint16_t spannung_wrd;       		// Ganzes Spannungs-Wort
uint16_t spannung_anz;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz1;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz2;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz3;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz4;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz5;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz6;       		// Ganzes Spannungs-Anzeige
uint16_t spannung_anz7;       		// Ganzes Spannungs-Anzeige

uint16_t msb_power;        			// Oberes Power-Byte
uint16_t lsb_power;        			// Unteres Power-Byte
uint16_t power_wrd;       			// Ganzes Power-Wort
uint16_t power_anz;       			// Ganzes Power-Anzeige

char Buffer[30];         			// Umwandlungs-Variable fr LCD Anzeige

void anzeige_1(void)				// Anzeuge erster Text
  {
    lcd_printlc(1,2,"Strommessung"); 		// Zeile 1
    lcd_printlc(2,3,"Programm 2");			// Zeile 2
    lcd_printlc(3,1,"mit INA219 Modul");	// Zeile 3
    lcd_printlc(4,5,"(by AS)");			// Zeile 4
  }

void anzeige_2(void)					// Anzeuge zweiter Text
  {
	lcd_printlc(1,1,"U Shunt");			// Zeile 2
	lcd_printlc(1,15,"mV");				// Zeile 2
	
	lcd_printlc(2,1,"U Bus:");			// Zeile 3
	lcd_printlc(2,16,"V");				// Zeile 3
	lcd_printlc(2,11,",");				// Zeile 3
	
	lcd_printlc(3,1,"Strom");			// Zeile 4
	lcd_printlc(3,15,"mA");				// Zeile 2
	
	lcd_printlc(4,1,"Power");			// Zeile 4
	lcd_printlc(4,15,"mW");				// Zeile 2
  }

int main(void)
  {
	cli();                     			// Interrupts deaktiviert
	i2c_init();                			// Starte I2C Bus$
	lcd_init();                			// Starte I2CLCD
	
	// Display Befehle
	lcd_command(LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKINGOFF);
	lcd_command(LCD_CLEAR);        		// Leere Display
	_delay_ms(2);		      			// Warte 2ms	
	anzeige_1();
	_delay_ms(3000);
	lcd_command(LCD_CLEAR);    			// Leere Display
	_delay_ms(2);		      			// Warte 2ms
	anzeige_2();
	_delay_ms(100);		       			// Warte 100ms
	
	// Kalibrierung eingabe
	i2c_start(adr_ina219);					// Angabe Adresse
	i2c_write(0x05);						// Angabe Calibration
	i2c_write (calibration >> 8);				// MSB
	i2c_write(calibration & 0xFF );				// LSB
	i2c_stop();
	
	while(1)
	  {
		// Shunt voltage 01
		i2c_start(adr_ina219);			// Angabe Adresse
		i2c_write(01);					// Register Strom	
		i2c_stop();
		i2c_start(adr_ina219 +1);		// Auslesen Adresse + 1
		msb_strom = i2c_readAck();		//...speichere oberes Bit
		lsb_strom = i2c_readNak();		//...speichere unteres Bit
		i2c_stop();
		strom_wrd = (msb_strom << 8 | lsb_strom);  // Zusammensetzung von o. & u. Byte
		strom_anz = strom_wrd / 8;
		
		if (strom_anz == 8191)
		  {
			strom_anz = 0;
		  }
		
		itoa(strom_anz, Buffer, 10 );
		lcd_printlc(1,10,"     ");
		lcd_printlc(1,10,Buffer);
		
		// Bus voltage 02
		i2c_start(adr_ina219);					// Angabe Adresse
		i2c_write(0x02);						// Register Spannung
		i2c_stop();
		i2c_start(adr_ina219 +1);				// Auslesen Adresse + 1
		msb_spannung = i2c_readAck();			//...speichere oberes Bit
		lsb_spannung = i2c_readNak();			//...speichere unteres Bit
		i2c_stop();
		spannung_wrd = (msb_spannung << 8 | lsb_spannung);  // Zusammensetzung von o. & u. Byte
		spannung_anz = spannung_wrd / 2;		// Berechnung Wert Spannung / 2
		
		spannung_anz1 = spannung_anz % 10;		// 4. Zahl Berechnung
		spannung_anz2 = spannung_anz / 10;
		spannung_anz3 = spannung_anz2 % 10;		// 3. Zahl Berechnung
		spannung_anz4 = spannung_anz2 / 10;
		spannung_anz5 = spannung_anz4 % 10;		// 2. Zahl Berechnung
		spannung_anz6 = spannung_anz4 / 10;
		spannung_anz7 = spannung_anz6 % 10;		// 1. Zahl Berechnung
		
		itoa(spannung_anz7, Buffer, 10 );		// 1. Zahl Anzeige
		lcd_printlc(2,10," ");
		lcd_printlc(2,10,Buffer);
		
		itoa(spannung_anz5, Buffer, 10 );		// 2. Zahl Anzeige
		lcd_printlc(2,12," ");
		lcd_printlc(2,12,Buffer);
		
		itoa(spannung_anz3, Buffer, 10 );		// 3. Zahl Anzeige
		lcd_printlc(2,13," ");
		lcd_printlc(2,13,Buffer);
		
		itoa(spannung_anz1, Buffer, 10 );		// 4. Zahl Anzeige
		lcd_printlc(2,14," ");
		lcd_printlc(2,14,Buffer);
		
		// Current 04
		i2c_start(adr_ina219);					// Angabe Adresse
		i2c_write(04);							// Register Strom		
		i2c_stop();
		i2c_start(adr_ina219 +1);				// Auslesen Adresse + 1
		msb_strom = i2c_readAck();				//...speichere oberes Bit
		lsb_strom = i2c_readNak();				//...speichere unteres Bit
		i2c_stop();
		strom_wrd = (msb_strom << 8 | lsb_strom);  // Zusammensetzung von o. & u. Byte
		strom_anz = strom_wrd / 8;				// Berechnung Wert Strom / 8
		
		if (strom_anz == 8191)
		  {
			strom_anz = 0;
		  }
		
		itoa(strom_anz, Buffer, 10 );
		lcd_printlc(3,10,"     ");
		lcd_printlc(3,10,Buffer);
		
		// Power 03
		i2c_start(adr_ina219);					// Angabe Adresse
		i2c_write(0x03);						// Register Power
		i2c_stop();
		i2c_start(adr_ina219 +1);				// Auslesen Adresse + 1
		msb_power = i2c_readAck();				//...speichere oberes Bit
		lsb_power = i2c_readNak();				//...speichere unteres Bit
		i2c_stop();
		power_wrd = (msb_power << 8 | lsb_power);  // Zusammensetzung von o. & u. Byte
		power_anz = power_wrd * 2;				// Wert Spannung / 2 100  muss alt sein 4
		itoa(power_anz, Buffer, 10 );
		lcd_printlc(4,10,"     ");
		lcd_printlc(4,10,Buffer);
      }
  }
		
		
		





